/*
 * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
 * All Rights Reserved.
 *
 * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
 * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
 * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
 * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
 * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
 * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
 * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
 * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
 * FOREGOING.
 */

package org.broad.igv.main;

import org.apache.log4j.Logger;
import org.broad.igv.Globals;
import org.broad.igv.exceptions.DataLoadException;
import org.broad.igv.ui.WaitCursorManager;
import org.broad.igv.util.NamedRunnable;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

public class BatchRunner implements NamedRunnable {
    private static Logger log = Logger.getLogger(BatchRunner.class);

    String inputFile;

    public BatchRunner(final String inputFile) {

        this.inputFile = inputFile;

    }

    public String getName() {
        return "batchExecution";  //To change body of implemented methods use File | Settings | File Templates.
    }

    public void run() {
        String inLine;
        CommandExecutor cmdExe = new CommandExecutor();
        Globals.setSuppress(true);
        Globals.batch = true;

        WaitCursorManager.CursorToken cursorToken = null;
        try {
            cursorToken = WaitCursorManager.showWaitCursor();
            File inFile = new File(inputFile);
            if (inFile.isFile()) {
                BufferedReader bfr = new BufferedReader(new FileReader(inputFile));

                while ((inLine = bfr.readLine()) != null) {
                    if (!(inLine.startsWith("#") || inLine.startsWith("//"))) {
                        log.info("Executing Command: " + inLine);
                        cmdExe.execute(inLine);
                    }
                }
                bfr.close();
            }
        } catch (IOException ioe) {
            throw new DataLoadException(ioe.getMessage(), inputFile);
        } finally {
            Globals.setSuppress(false);
            Globals.batch = false;
            if (cursorToken != null) WaitCursorManager.removeWaitCursor(cursorToken);
        }//To change body of implemented methods use File | Settings | File Templates.
    }

}
