/*
 * Copyright (c) 2007-2010 by The Broad Institute, Inc. and the Massachusetts Institute of Technology.
 * All Rights Reserved.
 *
 * This software is licensed under the terms of the GNU Lesser General Public License (LGPL), Version 2.1 which
 * is available at http://www.opensource.org/licenses/lgpl-2.1.php.
 *
 * THE SOFTWARE IS PROVIDED "AS IS." THE BROAD AND MIT MAKE NO REPRESENTATIONS OR WARRANTIES OF
 * ANY KIND CONCERNING THE SOFTWARE, EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION, WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT
 * OR OTHER DEFECTS, WHETHER OR NOT DISCOVERABLE.  IN NO EVENT SHALL THE BROAD OR MIT, OR THEIR
 * RESPECTIVE TRUSTEES, DIRECTORS, OFFICERS, EMPLOYEES, AND AFFILIATES BE LIABLE FOR ANY DAMAGES OF
 * ANY KIND, INCLUDING, WITHOUT LIMITATION, INCIDENTAL OR CONSEQUENTIAL DAMAGES, ECONOMIC
 * DAMAGES OR INJURY TO PROPERTY AND LOST PROFITS, REGARDLESS OF WHETHER THE BROAD OR MIT SHALL
 * BE ADVISED, SHALL HAVE OTHER REASON TO KNOW, OR IN FACT SHALL KNOW OF THE POSSIBILITY OF THE
 * FOREGOING.
 */
package org.broad.igv.sam.reader;

import net.sf.samtools.SAMFileReader;
import org.broad.igv.util.ResourceLocator;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * @author jrobinso
 */
public class SamQueryReaderFactory {

    static {
        SAMFileReader.setDefaultValidationStringency(SAMFileReader.ValidationStringency.SILENT);
    }

    public static AlignmentQueryReader getReader(String path, boolean cache) {
        return getReader(new ResourceLocator(path), cache);
    }

    public static AlignmentQueryReader getReader(ResourceLocator locator) {
        return getReader(locator, true);
    }

    public static AlignmentQueryReader getReader(ResourceLocator locator, boolean cache) {

        String path = locator.getPath().toLowerCase();


        AlignmentQueryReader reader = null;

        if (locator.isLocal()) {
            File samFile = new File(locator.getPath());
            if (path.endsWith(".bam")) {
                reader = new BAMQueryReader(samFile);
            } else if (path.endsWith(".sam")) {
                //return new SamInMemoryQueryReader(samFile);
                reader = new SamQueryTextReader(samFile);
            } else if (path.endsWith("sorted.txt")
                    || path.endsWith(".aligned")
                    || path.endsWith(".aligned.txt")
                    || path.endsWith("bedz")
                    || path.endsWith("bed")) {
                reader = new GeraldQueryReader(samFile);
            } else {
                throw new RuntimeException("Unknown file type: " + locator.getPath());
            }
        } else {
            if (locator.getPath().toLowerCase().startsWith("http") ||
                  locator.getPath().toLowerCase().startsWith("ftp")  ) {
                try {
                    URL url = new URL(locator.getPath());
                    reader = new BAMHttpQueryReader(url);
                }
                catch (MalformedURLException e) {
                    // todo log
                    e.printStackTrace();
                    return null;

                }
            } else {
                reader = new BAMRemoteQueryReader(locator);
            }
        }

        return cache ? new CachingQueryReader(reader) : reader;
    }
}

